@@ -35,6 +35,7 @@
 
 static void expand_macro __P ((symbol *, read_type));
 static void expand_unknown_tag __P ((char *, read_type));
+static void expand_entity __P ((symbol *, read_type));
 static void expand_token __P ((struct obstack *, read_type, token_type, token_data *));
 
 /* Current recursion level in expand_macro ().  */
@@ -73,6 +74,7 @@ expand_token (struct obstack *obs, read_type expansion, token_type t,
         token_data *td)
 {
   symbol *sym;
+  char *append_semicolon;
   char *text = TOKEN_DATA_TEXT (td);
 
   switch (t)
@@ -95,10 +97,13 @@ expand_token (struct obstack *obs, read_type expansion, token_type t,
       break;
 
     case TOKEN_WORD:
-      if (IS_ESCAPE(*text))
+      if (IS_TAG(*text))
         text++;
+      else
+        MP4HERROR ((warning_status, 0,
+          _("INTERNAL ERROR: macro has no leading '<' in expand_token ()")));
 
-      /* macro names must begin with a letter, an underscore or a %.
+      /* macro names must begin with a letter or an underscore.
          If another character is found, this string is not a
          macro.   */
 
@@ -130,6 +135,40 @@ expand_token (struct obstack *obs, read_type expansion, token_type t,
         shipout_text (obs, TOKEN_DATA_TEXT (td));
       break;
 
+    case TOKEN_ENTITY:
+      /* entity names must begin with a letter or an underscore.
+         If another character is found, this string is not an
+         entity.   */
+
+      if (IS_ENTITY(*text))
+        text++;
+      else
+        MP4HERROR ((warning_status, 0,
+          _("INTERNAL ERROR: entity has no leading '&' in expand_token ()")));
+
+      if (IS_ALPHA (*text))
+        {
+          append_semicolon = NULL;
+          if (LAST_CHAR (text) == ';')
+            {
+              append_semicolon = text + strlen (text) - 1;
+              *append_semicolon = '\0';
+            }
+
+          sym = lookup_entity (text, SYMBOL_LOOKUP);
+          if (sym == NULL || SYMBOL_TYPE (sym) == TOKEN_VOID)
+            {
+              if (append_semicolon)
+                *(append_semicolon) = ';';
+              shipout_text (obs, TOKEN_DATA_TEXT (td));
+            }
+          else
+            expand_entity (sym, expansion);
+        }
+      else
+        shipout_text (obs, TOKEN_DATA_TEXT (td));
+      break;
+
     default:
       MP4HERROR ((warning_status, 0,
         _("INTERNAL ERROR: Bad token type in expand_token ()")));
@@ -248,6 +287,7 @@ expand_argument (struct obstack *obs, read_type expansion, token_data *argp,
           break;
 
         case TOKEN_WORD:
+        case TOKEN_ENTITY:
           expand_token (obs, expansion, t, &td);
           break;
 
@@ -292,14 +332,15 @@ collect_arguments (char *symbol_name, read_type expansion,
   char last_char = ' ';
 
   TOKEN_DATA_TYPE (&td) = TOKEN_TEXT;
-  TOKEN_DATA_TEXT (&td) = xstrdup (symbol_name);
+  TOKEN_DATA_TEXT (&td) = symbol_name;
   tdp = (token_data *) obstack_copy (arguments, (voidstar) &td, sizeof (td));
   obstack_grow (argptr, (voidstar) &tdp, sizeof (tdp));
+  TOKEN_DATA_TYPE (&td) = TOKEN_VOID;
 
   ch = peek_input ();
   if (IS_CLOSE (ch))
     (void) next_token (&td, READ_BODY, FALSE);
-  else if (IS_SPACE(ch) || IS_ESCAPE(ch) || IS_SLASH(ch))
+  else if (IS_SPACE(ch) || IS_TAG(ch) || IS_SLASH(ch))
     {
       do
         {
@@ -374,6 +415,7 @@ collect_body (char *symbol_name, read_type expansion,
         case TOKEN_SIMPLE:
         case TOKEN_SPACE:
         case TOKEN_STRING:
+        case TOKEN_ENTITY:
           if (expansion_level > 0 && t == TOKEN_QUOTED)
             obstack_1grow (bodyptr, CHAR_LQUOTE);
           shipout_text (bodyptr, text);
@@ -382,7 +424,7 @@ collect_body (char *symbol_name, read_type expansion,
           break;
 
         case TOKEN_WORD:
-          if (IS_ESCAPE(*text))
+          if (IS_TAG(*text))
             text++;
 
           if (*text == '/')
@@ -533,12 +575,15 @@ expand_macro (symbol *sym, read_type expansion)
   read_type attr_expansion;
 
   expansion_level++;
-  array_current_line[expansion_level] = current_line;
   if (expansion_level > nesting_limit)
     MP4HERROR ((EXIT_FAILURE, 0,
       _("ERROR: Recursion limit of %d exceeded, use -L<N> to change it"),
            nesting_limit));
 
+  array_current_line[expansion_level] = current_line;
+  xfree((voidstar) array_current_file[expansion_level]);
+  array_current_file[expansion_level] = xstrdup(current_file);
+
   macro_call_id++;
   my_call_id = macro_call_id;
 
@@ -573,7 +618,7 @@ expand_macro (symbol *sym, read_type expansion)
       if (SYMBOL_CONTAINER (sym))
         {
           TOKEN_DATA_TYPE (&td) = TOKEN_TEXT;
-          TOKEN_DATA_TEXT (&td) = xstrdup ("");
+          TOKEN_DATA_TEXT (&td) = "";
           tdp = (token_data *) obstack_copy (&arguments,
                   (voidstar) &td, sizeof (td));
           obstack_grow (&argptr, (voidstar) &tdp, sizeof (tdp));
@@ -671,6 +716,14 @@ expand_unknown_tag (char *name, read_type expansion)
   boolean slash, single;
 
   expansion_level++;
+  if (expansion_level > nesting_limit)
+    MP4HERROR ((EXIT_FAILURE, 0,
+      _("ERROR: Recursion limit of %d exceeded, use -L<N> to change it"),
+           nesting_limit));
+
+  array_current_line[expansion_level] = current_line;
+  xfree((voidstar) array_current_file[expansion_level]);
+  array_current_file[expansion_level] = xstrdup(current_file);
 
   symbol_name = xstrdup (name);
 
@@ -768,3 +821,53 @@ expand_unknown_tag (char *name, read_type expansion)
 
   xfree ((voidstar) symbol_name);
 }
+
+
+/*-------------------------------------------------------------------------.
+| The entity expansion is handled by expand_entity ().  Entities are       |
+| treated similarly to macros, they just do not have arguments and the     |
+| symbol text is output directly.                                          |
+|                                                                          |
+| Expand_entity () is not recursive.                                       |
+`-------------------------------------------------------------------------*/
+
+static void
+expand_entity (symbol *sym, read_type expansion)
+{
+  struct obstack *obs_expansion;
+  const char *expanded;
+  boolean traced;
+  int my_call_id;
+
+  expansion_level++;
+  if (expansion_level > nesting_limit)
+    MP4HERROR ((EXIT_FAILURE, 0,
+      _("ERROR: Recursion limit of %d exceeded, use -L<N> to change it"),
+           nesting_limit));
+
+  array_current_line[expansion_level] = current_line;
+  xfree((voidstar) array_current_file[expansion_level]);
+  array_current_file[expansion_level] = xstrdup(current_file);
+
+  macro_call_id++;
+  my_call_id = macro_call_id;
+
+  traced = (boolean) ((debug_level & DEBUG_TRACE_ALL) || SYMBOL_TRACED (sym));
+
+  if (traced && (debug_level & DEBUG_TRACE_CALL))
+    trace_prepre (SYMBOL_NAME (sym), my_call_id);
+
+  obs_expansion = push_string_init ();
+  if (expansion_level > 1)
+    obstack_1grow (obs_expansion, CHAR_BGROUP);
+  shipout_string (obs_expansion, SYMBOL_TEXT (sym), 0);
+  if (expansion_level > 1)
+    obstack_1grow (obs_expansion, CHAR_EGROUP);
+  expanded = push_string_finish (expansion);
+
+  if (traced)
+    trace_post (SYMBOL_NAME (sym), my_call_id, 0, NULL, expanded);
+
+  --expansion_level;
+}
+ 
