@@ -139,6 +139,9 @@ Preprocessor features:\n\
       fputs (_("\
 \n\
 Parser features:\n\
+  -c, --caseless=NUMBER        set tags (1), variables (2) or entities (4)\n\
+                               case insensitive.  Default value is 3, i.e.\n\
+                               only entities are case sensitive\n\
   -e, --encoding=NAME          specify document encoding\n\
   -X, --expansion=NUMBER       set parser behaviour according to the bits\n\
                                of NUMBER, with (star marks current flags)\n\
@@ -219,6 +222,7 @@ static const struct option long_options[] =
   {"synclines", no_argument, NULL, 's'},
   {"safety-level", required_argument, NULL, 'S'},
   {"encoding", required_argument, NULL, 'e'},
+  {"caseless", required_argument, NULL, 'c'},
 
   {"help", no_argument, NULL, 'h'},
   {"version", no_argument, NULL, 'V'},
@@ -231,7 +235,7 @@ static const struct option long_options[] =
   { 0, 0, 0, 0 },
 };
 
-#define OPTSTRING "D:e:EF:H:I:L:QR:U:X:d:hl:o:sS:Ot:V"
+#define OPTSTRING "c:D:e:EF:H:I:L:QR:U:X:d:hl:o:sS:Ot:V"
 
 int
 main (int argc, char *const *argv)
@@ -244,6 +248,7 @@ main (int argc, char *const *argv)
   macro_definition *defines;
   FILE *fp;
   char *filename;
+  int caseless = CASELESS_DEFAULT;
 
   program_name = argv[0];
 
@@ -264,6 +269,12 @@ main (int argc, char *const *argv)
       case 0:
         break;
 
+      case 'c':
+        caseless = atoi (optarg);
+        if (caseless <= 0)
+          caseless = CASELESS_DEFAULT;
+        break;
+
       case 'D':
       case 'U':
       case 't':
@@ -386,7 +397,12 @@ main (int argc, char *const *argv)
   output_init ();
   include_env_init ();
   symtab_init ();
-  locale_init ();
+  break_init ();
+  caseless_init (caseless);
+#ifdef HAVE_LOCALE_H
+  locale_init (LC_ALL, NULL);
+#endif
+  pcre_init ();
 
   if (frozen_file_to_read)
     reload_frozen_state (frozen_file_to_read);
@@ -457,7 +473,9 @@ main (int argc, char *const *argv)
             if (fp == NULL)
               {
                 error (0, errno, argv[optind]);
-                abort ();
+                MP4HERROR ((warning_status, 0,
+                   _("%s: file skipped"), argv[optind]));
+                continue;
               }
             else
               {
@@ -482,9 +500,12 @@ main (int argc, char *const *argv)
   /* Free memory */
   input_deallocate ();
   debug_deallocate ();
+  include_deallocate ();
   output_deallocate ();
   symtab_deallocate ();
+  break_deallocate ();
   builtin_deallocate ();
+  pcre_deallocate ();
 
   xfree ((voidstar) current_file);
 
