@@ -103,11 +103,10 @@ extern int errno;
 /* If FALSE is defined, we presume TRUE is defined too.  In this case,
    merely typedef boolean as being int.  Or else, define these all.  */
 #ifndef FALSE
-/* Do not use `enum boolean': this tag is used in SVR4 <sys/types.h>.  */
-typedef enum { FALSE = 0, TRUE = 1 } boolean;
-#else
-typedef int boolean;
+# define FALSE 0
+# define TRUE 1
 #endif
+typedef int boolean;
 
 char *mktemp ();
 
@@ -138,12 +137,64 @@ char *mktemp ();
 
 /* Various declarations.  */
 
-struct string
+struct _string
   {
     char *string;               /* characters of the string */
     size_t length;              /* length of the string */
   };
-typedef struct string STRING;
+typedef struct _string STRING;
+
+/* Those must come first.  */
+typedef void builtin_func ();
+
+/* Various different token types.  */
+enum _token_type
+{
+  TOKEN_EOF,                    /* end of file */
+  TOKEN_NONE,                   /* discardable token */
+  TOKEN_STRING,                 /* a string */
+  TOKEN_QUOTED,                 /* a quoted string */
+  TOKEN_QUOTE,                  /* begin delimiter of a quoted string
+                                   to expand */
+  TOKEN_BGROUP,                 /* begin group */
+  TOKEN_EGROUP,                 /* end group */
+  TOKEN_SPACE,                  /* whitespace */
+  TOKEN_WORD,                   /* an identifier */
+  TOKEN_ENTITY,                 /* an entity */
+  TOKEN_SIMPLE,                 /* a single character */
+  TOKEN_MACDEF                  /* a macros definition (see "defn") */
+};
+typedef enum _token_type token_type;
+
+/* The data for a token, a macro argument, and a macro definition.  */
+enum _token_data_type
+{
+  TOKEN_VOID,
+  TOKEN_TEXT,
+  TOKEN_FUNC
+};
+typedef enum _token_data_type token_data_type;
+
+struct _token_data
+{
+  token_data_type type;
+  union
+    {
+      struct
+        {
+          char *text;
+        }
+      u_t;
+      struct
+        {
+          builtin_func *func;
+          boolean traced;
+        }
+      u_f;
+    }
+  u;
+};
+typedef struct _token_data token_data;
 
 /* Memory allocation.  */
 voidstar xmalloc __P ((size_t));
@@ -156,10 +207,6 @@ char *xstrdup __P ((const char *));
 /* Other library routines.  */
 void error __P ((int, int, const char *, ...));
 
-/* Those must come first.  */
-typedef void builtin_func ();
-typedef struct token_data token_data;
-
 
 /* File: mp4h.c  --- global definitions.  */
 
@@ -275,51 +322,6 @@ void trace_post __P ((const char *, int, int, token_data **, const char *));
 
 /* File: input.c  --- lexical definitions.  */
 
-/* Various different token types.  */
-enum token_type
-{
-  TOKEN_EOF,                    /* end of file */
-  TOKEN_NONE,                   /* discardable token */
-  TOKEN_STRING,                 /* a string */
-  TOKEN_QUOTED,                 /* a quoted string */
-  TOKEN_QUOTE,                  /* begin delimiter of a quoted string
-                                   to expand */
-  TOKEN_BGROUP,                 /* begin group */
-  TOKEN_EGROUP,                 /* end group */
-  TOKEN_SPACE,                  /* whitespace */
-  TOKEN_WORD,                   /* an identifier */
-  TOKEN_SIMPLE,                 /* a single character */
-  TOKEN_MACDEF                  /* a macros definition (see "defn") */
-};
-
-/* The data for a token, a macro argument, and a macro definition.  */
-enum token_data_type
-{
-  TOKEN_VOID,
-  TOKEN_TEXT,
-  TOKEN_FUNC
-};
-
-struct token_data
-{
-  enum token_data_type type;
-  union
-    {
-      struct
-        {
-          char *text;
-        }
-      u_t;
-      struct
-        {
-          builtin_func *func;
-          boolean traced;
-        }
-      u_f;
-    }
-  u;
-};
-
 #define TOKEN_DATA_TYPE(Td)         ((Td)->type)
 #define TOKEN_DATA_TEXT(Td)         ((Td)->u.u_t.text)
 #define TOKEN_DATA_FUNC(Td)         ((Td)->u.u_f.func)
@@ -361,8 +363,6 @@ struct token_data
 
 extern int exp_flags;
 
-typedef enum token_type token_type;
-typedef enum token_data_type token_data_type;
 typedef int read_type;
 
 void input_init __P ((void));
@@ -390,9 +390,13 @@ void read_file_verbatim __P ((struct obstack *));
 /* current input file, and line */
 extern char *current_file;
 extern int current_line;
+extern char **array_current_file;
 extern int *array_current_line;
-#define CURRENT_FILE_LINE current_file, (expansion_level == 0 ? current_line: \
-                                        array_current_line[expansion_level])
+#define CURRENT_FILE_LINE                                               \
+        (expansion_level == 0 ? current_file :                          \
+                                array_current_file[expansion_level]),   \
+        (expansion_level == 0 ? current_line :                          \
+                                array_current_line[expansion_level])
 
 /* Begin and end quote */
 extern STRING lquote, rquote;
@@ -426,9 +430,6 @@ extern unsigned short syntax_table[256];
 
 #define SYNTAX_IGNORE   (0x0001)
 #define SYNTAX_SPACE    (0x0002)
-#define SYNTAX_CLOSE    (0x0004)
-#define SYNTAX_ACTIVE   (0x0007)
-#define SYNTAX_ESCAPE   (0x0008)
 #define SYNTAX_GROUP    (0x0009)
 
 /* These are values to be assigned to syntax table entries, but they are
@@ -444,9 +445,6 @@ extern unsigned short syntax_table[256];
 #define IS_OTHER(ch)  ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_OTHER)
 #define IS_IGNORE(ch) ((syntax_table[(int)(ch)]) == SYNTAX_IGNORE)
 #define IS_SPACE(ch)  ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_SPACE)
-#define IS_CLOSE(ch)  ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_CLOSE)
-#define IS_ACTIVE(ch) ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_ACTIVE)
-#define IS_ESCAPE(ch) ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_ESCAPE)
 #define IS_ALPHA(ch)  ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_ALPHA)
 #define IS_NUM(ch)    ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_NUM)
 #define IS_ALNUM(ch)  ((((syntax_table[(int)(ch)]) & SYNTAX_ALNUM) != 0) \
@@ -458,6 +456,9 @@ extern unsigned short syntax_table[256];
 #define IS_RQUOTE(ch) (ch == CHAR_RQUOTE)
 #define IS_GROUP(ch)  ((syntax_table[(int)(ch)]&SYNTAX_VALUE) == SYNTAX_GROUP)
 #define IS_SLASH(ch)  (ch == CHAR_SLASH || ch == '/')
+#define IS_TAG(ch)    (ch == '<')
+#define IS_CLOSE(ch)  (ch == '>')
+#define IS_ENTITY(ch) (ch == '&')
 
 void set_syntax __P ((int, const char *));
 void set_syntax_internal __P ((int, int));
@@ -480,8 +481,11 @@ void remove_special_chars __P ((char *, boolean));
 
 /* File symtab.c  --- symbol table definitions.  */
 
+/*  Default case sensitiveness  */
+#define CASELESS_DEFAULT 3
+
 /* Operation modes for lookup_symbol ().  */
-enum symbol_lookup
+enum _symbol_lookup
 {
   SYMBOL_LOOKUP,
   SYMBOL_INSERT,
@@ -489,9 +493,9 @@ enum symbol_lookup
 };
 
 /* Symbol table entry.  */
-struct symbol
+struct _symbol
 {
-  struct symbol *next;
+  struct _symbol *next;
   boolean traced;
   boolean container;
   boolean expand_args;
@@ -513,8 +517,8 @@ struct symbol
 #define SYMBOL_TEXT(S)          (TOKEN_DATA_TEXT (&(S)->data))
 #define SYMBOL_FUNC(S)          (TOKEN_DATA_FUNC (&(S)->data))
 
-typedef enum symbol_lookup symbol_lookup;
-typedef struct symbol symbol;
+typedef enum _symbol_lookup symbol_lookup;
+typedef struct _symbol symbol;
 typedef void hack_symbol ();
 
 #define HASHMAX 509             /* default, overridden by -Hsize */
@@ -526,7 +530,9 @@ extern symbol **symtab;
 
 void symtab_init __P ((void));
 void symtab_deallocate __P ((void));
+void caseless_init __P ((int));
 symbol *lookup_symbol __P ((const char *, symbol_lookup));
+symbol *lookup_entity __P ((const char *, symbol_lookup));
 symbol *lookup_variable __P ((const char *, symbol_lookup));
 symbol *lookup_file __P ((const char *, symbol_lookup));
 void hack_all_symbols __P ((hack_symbol *, const char *));
@@ -543,14 +549,14 @@ extern int expansion_level;
 
 /* File: builtin.c  --- builtins.  */
 
-enum encoding_type
+enum _encoding_type
 {
   ENCODING_8BIT,                /* 1-byte char */
   ENCODING_UTF8                 /* UTF-8 */
 };
-typedef enum encoding_type encoding_type;
+typedef enum _encoding_type encoding_type;
 
-struct builtin
+struct _builtin
 {
   const char *name;
   boolean container;
@@ -558,7 +564,7 @@ struct builtin
   builtin_func *func;
 };
 
-typedef struct builtin builtin;
+typedef struct _builtin builtin;
 
 extern boolean visible_quotes;
 
@@ -568,13 +574,16 @@ extern int safety_level;
 /* Document encoding */
 extern encoding_type document_encoding;
 
-void locale_init __P ((void));
+void locale_init __P ((int, char *));
+void pcre_init __P ((void));
+void pcre_deallocate __P ((void));
 void initialize_builtin __P ((symbol *));
 void builtin_init __P ((void));
 void builtin_deallocate __P ((void));
 void clear_tag_attr __P ((void));
 void define_builtin __P ((const char *, const builtin *, boolean));
-void init_break __P ((void));
+void break_init __P ((void));
+void break_deallocate __P ((void));
 void define_user_macro __P ((const char *, char *, symbol_lookup,
                              boolean, boolean, boolean));
 void undivert_all __P ((void));
@@ -601,6 +610,7 @@ const char * predefined_attribute __P ((const char *, int *, token_data **, bool
 
 void include_init __P ((void));
 void include_env_init __P ((void));
+void include_deallocate __P ((void));
 void add_include_directory __P ((const char *));
 FILE *path_search __P ((const char *, char **));
 
@@ -697,7 +707,7 @@ void module_unload_all __P ((void));
 #define ARG(i)  (i<argc ? TOKEN_DATA_TEXT (argv[i]) : "")
 #define ARGBODY (TOKEN_DATA_TEXT (argv[argc]))
 
-enum mathop_type
+enum _mathop_type
 {
   MATHOP_ADD,                   /* addition */
   MATHOP_SUB,                   /* substraction */
@@ -708,7 +718,7 @@ enum mathop_type
   MATHOP_MOD                    /* modulus */
 };
 
-enum mathrel_type
+enum _mathrel_type
 {
   MATHREL_GT,
   MATHREL_LT,
@@ -716,8 +726,8 @@ enum mathrel_type
   MATHREL_NEQ 
 };
 
-typedef enum mathop_type mathop_type;
-typedef enum mathrel_type mathrel_type;
+typedef enum _mathop_type mathop_type;
+typedef enum _mathrel_type mathrel_type;
 
 typedef struct var_stack var_stack;
 struct var_stack
